﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Problems.DataAnalysis.Evaluators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.DataAnalysis.MultiVariate.Evaluators {
  public class OnlineMultiVariateEvaluator<T> : IMultiVariateOnlineEvaluator where T : IOnlineEvaluator, new() {
    private List<T> evaluators;

    public OnlineMultiVariateEvaluator() {

      Reset();
    }

    #region IMultiVariateOnlineEvaluator Members
    public double Value {
      get { return evaluators.Sum(e => e.Value); }
    }

    public void Add(double[] original, double[] estimated) {
      if (original.Length != estimated.Length) throw new ArgumentException("Number of elements of original and estimated doesn't match.");
      if (evaluators == null) InitializeEvaluators(original.Length);
      if (original.Length != evaluators.Count) throw new ArgumentException("Dimension doesn't match previous array dimensions.");
      for (int i = 0; i < original.Length; i++) {
        evaluators[i].Add(original[i], estimated[i]);
      }
    }

    private void InitializeEvaluators(int count) {
      evaluators = new List<T>();
      for (int i = 0; i < count; i++) {
        evaluators.Add(new T());
      }
    }

    public void Reset() {
      if (evaluators != null) {
        foreach (var evaluator in evaluators)
          evaluator.Reset();
      }
    }

    #endregion
  }
}
